/******************************************************************

	Command:  AREA -- Calculate close area command

	Purpose:  This program is used to calculate the total area
		  of selected objects.

	Programmed by Kang, Hsin-Min.
	Released in 1994 as a working example.
	Revised at Feb 09, 1998 -- Simplify the operation and rewrite
				   for standard command.
	Revised at Apr 22, 1998 Include Japenese Text in SJIS codes
	Revised at Aug 05, 1998 Modify Japenese Messages.

	Copyright(c) 1994, 1995, 1998 TCAM Development House, Taipei.
	All rights Reserved.

	Status: TwinCAD Standard Command, with full support from
		TCAM Development House.

    Comments:

	This program implements an equivalent ACAD's AREA command.

	If the area to find is not directly formed by close polylines
	and circles, use PAREA or BPOLY command instead.

*******************************************************************/
//#include "tclcad.h"
// Resolve definition from tclcad.h
#define T_POLY		0
#define T_CIRCLE	4
#define T_ELLIPSE	19

#define TGBIG5		1
#define TGGB2312	2
#define TGSJIS		5
#define TGKOREAN	3

char	szText[5][70];		// Our display text buffer
char	szAsciiText[][70]={	// Default English Display Text
/*  0 */   "\nArea -- Entity/Subtract/<Add Mode>: ",
/*  1 */   "\nArea -- Entity/Add/<Subtract Mode>: ",
/*  2 */   "\nArea -- Add/Subtract/<Entity>: ",
/*  3 */   "\n** Please select circle or closed Polyline. ",
/*  4 */   "\nArea=%14.6f, Total Area = %14.6f",
};
char	szBig5Text[][60]={	// Default Chinese Big-5 Display Text
/*  0 */   "\nnp -- :@Ϥ/:/<֥[Ϥn>: ",
/*  1 */   "\nnp -- :@Ϥ/:֥[/<Ϥn>: ",
/*  2 */   "\nnp -- :֥[/:/<@Ϥn>: ",
/*  3 */   "\n** пܶΫʳƽu.",
/*  4 */   "\n@n=%14.6f, `n = %14.6f ",
};
char	szGB2312Text[][70]={	// Default Chinese GB2312 Display Text
/*  0 */   "\n -- :һͼԪ/:۳/<ۼͼԪ>: ",
/*  1 */   "\n -- :һͼԪ/:ۼ/<۳ͼԪ>: ",
/*  2 */   "\n -- :ۼ/:۳/<һͼԪ>: ",
/*  3 */   "\n** ѡԲյĸ.",
/*  4 */   "\nһֵ=%14.6f,  = %14.6f ",
};
//
//	The following Japenese messages are translated by _____________.
//
char	szSJISText[][70]={    // Default Japenese SJIS Display Text
/*  0 */   "\nʐ -- E:}`/S:O/<A:ǉ>: ",
/*  1 */   "\nʐ -- E:}`/A:ǉ/<S:O>: ",
/*  2 */   "\nʐ -- A:ǉ/S:O/<E:}`>: ",
/*  3 */   "\n** ~͕|CIĉB ",
/*  4 */   "\nʐ=%14.6f, Sʐ = %14.6f",
};
//
//	The following Korean messages are translated by _____________.
//
char	szKoreanText[][70]={	// Default Korean Display Text
/*  0 */   "\nArea -- Entity/Subtract/<Add Mode>: ",
/*  1 */   "\nArea -- Entity/Add/<Subtract Mode>: ",
/*  2 */   "\nArea -- Add/Subtract/<Entity>: ",
/*  3 */   "\n** Please select circle or closed Polyline. ",
/*  4 */   "\nArea=%14.6f, Total Area = %14.6f",
};
COMMAND Area()
{
	double total, area;
	int    mode,i;	// Area mode

	switch(@tgftype) {
	  case TGBIG5:	  setdata(szText,szBig5Text);	  break;
	  case TGGB2312:  setdata(szText,szGB2312Text);   break;
	  case TGSJIS:	  setdata(szText,szSJISText);	  break;
	  case TGKOREAN:  setdata(szText,szKoreanText);   break;
	  default:	  setdata(szText,szAsciiText);	  break;
	}
	area = total = mode = 0;
	set_drag(-1);	// Turn off dragging, if it has been enabled.
	for(;;) {
	    switch(mode) {
	      case 1: i=getinput(szText[0],0,1,"E","S"); break;
	      case 2: i=getinput(szText[1],0,1,"A","E"); break;
	     default: i=getinput(szText[2],0,1,"A","S"); break;
	    }
	    if ( i <= 2 ) break; // Quit on ^C, space, value, point
	    if ( i > 3 ) {
		i -= 3;
		if ((mode==1)&&(i!=2)) i=i^1;
		if ((mode==2)&&(i!=1)) i=i^2;
		mode=i;
		continue;
	    }
	    if ( ent_ok(ent_main(e0)) ) e0 = ent_main(e0);
	    switch(ent_type(e0)) {
	      case T_POLY:
	      case T_CIRCLE:
		break;
	      case T_ELLIPSE:
		if ( abs(ent_read(e0).elps.span) == pi*2 )
		    break;
	      default:
		printf(szText[3]);
		continue;
	    }
	    area=ent_area(e0);
	    switch(mode) {
	      case 0: total =abs(area); break;
	      case 1: total+=abs(area); break;
	      case 2: total-=abs(area); break;
	    }
	    printf(szText[4],area,total);
	}
}
